#!/usr/local/bin/perl
# 
# Object class for manupulating AKAI S3000 program/sample.
# Copyright (c) 1997, Hiroyuki Ohsaki.
# All rights reserved.
# 
# $Id: AkaiSample.pm,v 1.3 1998/07/11 16:46:14 oosaki Exp $
# 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

package Synth::AkaiSample;

require 5.000;
use Synth::AkaiDisk;
use strict;

use vars qw(@ISA @EXPORT);

require Exporter;
@ISA = qw(Exporter);

sub new {
    my $class = shift;

    my $this = {};
    bless $this;
    $this;
}

sub write_common {
    my($this, %common) = @_;

    $this->{common} = { %common };
}

sub write_sample {
    my($this, %wave) = @_;

    $this->{sample}->{$wave{name}} = { %wave };
}

sub write_keygroup {
    my($this, %keygroup) = @_;

    if ($keygroup{low_key} < 24) {
	$keygroup{low_key} = 24;
    }
    push(@{$this->{keygroup}}, { %keygroup });
}

sub save_sample {
    my($this, $dir, %wave) = @_;

    my $header = '';
    for (<<EOF =~ /(\S+)/g) {	# 0x0000 - 0x00bf
03 01 3C 1D 13 18 0F 0A  0A 0A 0A 0A 0A 0A 0A 80
01 00 00 00 00 00 00 04  01 00 00 01 00 00 16 00
00 00 FF 00 00 00 C0 00  00 00 00 00 A8 00 00 00
0F 27 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 97 7A  16 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00  FF FF 44 AC 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
EOF
        $header .= pack('H2', $_);
    }
    substr($header, 0x02, 1) = pack('C', $wave{key});
    substr($header, 0x03, 12) 
	= pack('A12', Synth::AkaiDisk::ascii_to_akai($wave{name}));
    substr($header, 0x14, 2) = pack('v', $wave{tune} * 256);
    substr($header, 0x1a, 4) = pack('V', $wave{size});
    substr($header, 0x1e, 4) = pack('V', $wave{start});
    substr($header, 0x22, 4) = pack('V', $wave{end});
    substr($header, 0x26, 4) = pack('V', $wave{loop_start});
    substr($header, 0x2c, 4) = pack('V', $wave{loop_len});
    substr($header, 0x30, 2) = pack('v', $wave{loop_times});
    substr($header, 0x8a, 2) = pack('v', $wave{rate});
    
    my $name = $wave{name};
    $name =~ tr/A-Z /a-z_/;
    local(*OUT);
    open(OUT, ">$dir/$name.a3s") || die "$!: $dir/$name.a3s\n";
    binmode(OUT);
    print OUT $header;
    print OUT ${$wave{data}};
    close(OUT);
}

sub save_program {
    my($this, $dir) = @_;

    my %common = %{$this->{common}};
    my @keygroup = @{$this->{keygroup}};
    my $nkeygroup = @keygroup;
    my $buf;

    my $common = '';
    for (<<EOF =~ /(\S+)/g) {	# 0x0000 - 0x00bf
01 0C 90 1E 0F 1D 1E 0A  1A 1C 19 11 1C 0B 17 00
00 1F 01 18 7F 00 FF 63  00 50 14 00 00 01 63 00
00 32 00 00 14 0A 00 02  00 00 01 01 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 0A 0A
0A 00 00 00 00 00 32 00  00 02 00 00 08 06 0C 06
03 06 06 06 05 08 0A 0A  05 00 00 00 00 00 00 00
00 00 00 05 08 0E 00 08  08 00 00 00 00 4D 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00
EOF
        $common .= pack('H2', $_);
    }

    substr($common, 0x03, 12)
	= pack('A12', Synth::AkaiDisk::ascii_to_akai($common{name})); 
    substr($common, 0x0f, 1) = pack('C', $common{midi_prog});
    substr($common, 0x10, 1) = pack('C', $common{midi_chan});
    substr($common, 0x13, 1) = pack('C', $common{low_key});
    substr($common, 0x14, 1) = pack('C', $common{high_key});
    substr($common, 0x2a, 1) = pack('C', $nkeygroup);
    $buf .= $common;

    my $keygroup = '';
    for (<<EOF =~ /(\S+)/g) {	# 0x00c0 - 0x00e1
02 00 00 18 7F 00 00 50  00 09 00 0A 19 32 63 2D
00 00 00 00 00 32 63 2D  00 00 00 00 19 00 01 04
FF FF 1D 13 18 0F 0A 0A  0A 0A 0A 0A 0A 0A 00 7F
00 00 00 00 00 00 FF FF  3C 90 0A 0A 0A 0A 0A 0A
0A 0A 0A 0A 0A 0A 00 00  00 00 00 00 00 00 FF FF
FF FF 0A 0A 0A 0A 0A 0A  0A 0A 0A 0A 0A 0A 00 00
00 00 00 00 00 00 FF FF  FF FF 0A 0A 0A 0A 0A 0A
0A 0A 0A 0A 0A 0A 00 00  00 00 00 00 00 00 FF FF
FF FF 00 00 00 00 00 00  00 00 00 00 00 00 00 00
00 00 00 00 00 00 32 09  00 0A 00 00 63 32 63 00
FF 00 00 00 00 00 00 00  01 01 00 00 00 00 00 00
00 63 00 00 63 32 63 00  63 2D 00 00 00 00 00 19
EOF
        $keygroup .= pack('H2', $_);
    }

    for (0 .. $nkeygroup - 1) {
	my %key = %{$keygroup[$_]};
	
	# keygroup common
	substr($keygroup, 0x03, 1) = pack('C', $key{low_key});
	substr($keygroup, 0x04, 1) = pack('C', $key{high_key});
	substr($keygroup, 0x05, 2) = pack('v', $key{tune} * 256);

	# velocity zone 1
	substr($keygroup, 0x22, 12) 
	    = pack('A12', Synth::AkaiDisk::ascii_to_akai($key{name}));
	substr($keygroup, 0x2e, 1) = pack('C', $key{low_vel});
	substr($keygroup, 0x2f, 1) = pack('C', $key{high_vel});
	substr($keygroup, 0x32, 1) = pack('c', $key{vol_offset});
	substr($keygroup, 0x34, 1) = pack('c', $key{pan_offset});

	# more keygroup common
	substr($keygroup, 0x84, 1) = pack('C', $key{pitch});
	$buf .= $keygroup;
    }
    
    my $name = $common{name};
    $name =~ tr/A-Z /a-z_/;
    local(*OUT);
    open(OUT, ">$dir/$name.a3p") || return;
    binmode(OUT);
    print OUT $buf;
    close(OUT);
}    

sub save {
    my($this, $dir) = @_;

    my $name;
    for $name (keys %{$this->{sample}}) {
	$this->save_sample($dir, %{$this->{sample}{$name}});
    }
    $this->save_program($dir);
}

sub dump_wave {
    my($this, %wave) = @_;

    print <<EOF;
name       => $wave{name},
size       => $wave{size},
rate       => $wave{rate},
key        => $wave{key},
tune       => $wave{tune},
start      => $wave{start},
end        => $wave{end},
loop_start => $wave{loop_start},
loop_len   => $wave{loop_len},
loop_times => $wave{loop_times},
data       => $wave{data},

EOF
}
