#!/usr/local/bin/perl
# 
# Object class for manipulating RIFF WAVE file.
# Copyright (c) 1997, Hiroyuki Ohsaki.
# All rights reserved.
# 
# $Id: WaveSample.pm,v 1.2 1998/07/03 14:32:54 oosaki Exp $
# 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

package Synth::WaveSample;

require 5.000;
use POSIX;
use Symbol;
use File::Basename;
use strict;

use vars qw(@ISA @EXPORT);

require Exporter;
@ISA = qw(Exporter);

sub new {
    my $class = shift;

    my $this = {};
    bless $this;
    $this;
}

sub seek {
    my($this, $pos, $whence) = @_;

    $whence = 0 unless defined($whence);
    seek($this->{fh}, $this->{offset} + $pos, $whence) || return;
}

sub load {
    my($this, $file) = @_;

    $this->{file} = $file;

    $this->{fh} = gensym;
    open($this->{fh}, $file) || return;
    binmode($this->{fh});

    # check if file format is RIFF WAVE
    seek($this->{fh}, 4, 0);
    $_ = '';
    read($this->{fh}, $_, 8);
    my($size, $id) = unpack('LA4', $_);
    ($! = EIO, return) unless ($id eq 'WAVE');

    $this->{size} = $size;
    $this->{offset} = 12;
    1;
}

sub find_chunk {
    my($this, $id) = @_;

    my $pos = 0;
    while ($pos < $this->{size}) {
	$this->seek($pos);
	$_ = '';
	read($this->{fh}, $_, 8);
	my($type, $size) = unpack('A4L', $_);
	return ($pos + 8, $size) if ($type =~ $id);
	$pos += 8 + $size;
    }
    return ();
}

sub read_wave_header {
    my $this = shift;

    # search a format chunk
    my($pos, $size) = $this->find_chunk('fmt');
    return () unless $pos;

    # read format chunk
    $this->seek($pos);
    my $buf = '';
    read($this->{fh}, $buf, $size);
    my($tag, $chan, $rate, $average, $align, $bits)
	= unpack('SSLLSS', $buf);

    # search a data chunk
    my($pos, $size) = $this->find_chunk('data');

    (name    => basename($this->{file}, '.wav'),
     tag     => $tag,
     chan    => $chan,
     rate    => $rate,
     average => $average,
     align   => $align,
     bits    => $bits,
     address => $pos,
     size    => $size,
     );
}

sub read_wave_data {
    my $this = shift;

    my %wave = $this->read_wave_header;
    return undef unless (exists $wave{address});
    
    $this->seek($wave{address});
    my $data = '';
    read($this->{fh}, $data, $wave{size});
    return \$data;
}

sub dump_wave {
        my($this, %wave) = @_;

    print <<EOF;
name    => $wave{name}
tag     => $wave{tag}
chan    => $wave{chan}
rate    => $wave{rate}
average => $wave{average}
align   => $wave{align}
bits    => $wave{bits}
address => $wave{address}
size    => $wave{size}

EOF
}
